@@ -3,7 +3,7 @@ from .expression import *
 
 
 """
-    AST for parsed Python Query Batch.  Allows validation, normalization, 
+    AST for parsed Python Query Batch.  Allows validation, normalization,
     rewriting, and serialization.  Grammar is a strict subset of SQL-92.
     Lexer and parser token names borrowed from SparkSQL Grammar.
 """
@@ -22,7 +22,7 @@ class Batch(Sql):
 class Query(SqlRel):
     """A single query"""
 
-    def __init__(self, select, source, where, agg, having, order, limit) -> None:
+    def __init__(self, select, source, where, agg, having, order, limit, metadata=None) -> None:
         self.select = select
         self.source = source
         self.where = where
@@ -43,6 +43,9 @@ class Query(SqlRel):
         self.m_sym_dict = None
         self.m_symbols = None
 
+        if metadata:
+            self.load_symbols(metadata)
+
     def load_symbols(self, metadata):
         symbols = []
         relations = self.source.relations
@@ -64,16 +67,17 @@ class Query(SqlRel):
         self.m_symbols = symbols
         self.m_sym_dict = {}
 
-        tables = []
         for name, symbol in self.m_symbols:
-            for table in symbol.find_nodes(TableColumn):
-                tables.append(table)
             if name == "???":
                 continue
             if name in self.m_sym_dict:
                 raise ValueError("SELECT has duplicate column names: " + name)
             self.m_sym_dict[name] = symbol
 
+        tables = []
+        for t in self.find_nodes(Table):
+            tables.append(t.m_symbols[0][1])
+
         if len(tables) > 0:
             self.max_ids = max(tc.max_ids for tc in tables)
             self.sample_max_ids = any(tc.sample_max_ids for tc in tables)
@@ -251,7 +255,7 @@ class Relation(SqlRel):
             symbol
             for name, symbol in syms_b
             if (type(symbol) is TableColumn and symbol.compare.identifier_match(colname, name))
-            or name.lower() == colname.lower()
+            or name == colname
         ]
         if len(syms_c) == 1:
             return syms_c[0]
