@@ -10,6 +10,7 @@ from opendp.smartnoise.sql.reader import PandasReader
 
 from .dataset_adapter import DatasetAdapter
 
+
 def _make_doi_host(host, doi):
     doi = doi.replace("doi:", "")
     return "{}/api/access/datafile/:persistentId/?persistentId=doi:{}".format(host, doi)
@@ -29,6 +30,7 @@ def dataverse_loader(host, doi=None, token=None):
         stream.write(response.text)
     return pd.read_csv(path, sep="\t")
 
+
 def dataverse_uri_loader(dv_uri):
     uri = parser.urlparse(dv_uri)
     try:
@@ -39,14 +41,16 @@ def dataverse_uri_loader(dv_uri):
     except Exception as e:
         print(str(e))
 
-    doi_key = 'persistentId'
-    token_key = 'token'
-    kv_arr = [pair.split('=') for pair in kv]
+    doi_key = "persistentId"
+    token_key = "token"
+    kv_arr = [pair.split("=") for pair in kv]
     kv_dict = dict(kv_arr)
     doi = kv_dict[doi_key]
     token = kv_dict[token_key]
-    url = 'https://' + host + path + '?' + doi_key + '=' + doi;
-    response = requests.get(url, headers={'X-Dataverse-Key': '%s' % token}) if token else requests.get(url)
+    url = "https://" + host + path + "?" + doi_key + "=" + doi
+    response = (
+        requests.get(url, headers={"X-Dataverse-Key": "%s" % token}) if token else requests.get(url)
+    )
     response.raise_for_status()
 
     temp_dir = tempfile.gettempdir()
@@ -55,6 +59,7 @@ def dataverse_uri_loader(dv_uri):
         stream.write(response.text)
     return pd.read_csv(path, sep="\t")
 
+
 class DataverseAdapter(DatasetAdapter):
     KEY = "dataverse_details"
 
@@ -65,8 +70,9 @@ class DataverseAdapter(DatasetAdapter):
 
     @staticmethod
     def _load_df(dataset_document):
-        return dataverse_loader(dataset_document.dataverse_details.host,
-                                token=dataset_document.dataverse_details.token)
+        return dataverse_loader(
+            dataset_document.dataverse_details.host, token=dataset_document.dataverse_details.token
+        )
 
     @staticmethod
     def _load_metadata(dataset_document):
@@ -74,4 +80,7 @@ class DataverseAdapter(DatasetAdapter):
 
     @staticmethod
     def _load_reader(dataset_document):
-        return PandasReader(DataverseAdapter.load_df(dataset_document), DataverseAdapter.load_metadata(dataset_document))
+        return PandasReader(
+            DataverseAdapter.load_df(dataset_document),
+            DataverseAdapter.load_metadata(dataset_document),
+        )
