@@ -18,8 +18,12 @@ class LocalCSVAdapter(DatasetAdapter):
         return pd.read_csv(dataset_document.csv_details.local_path)
 
     def _load_metadata(dataset_document):
-        return CollectionMetadata.from_file(dataset_document.csv_details.local_path.split(".")[0] + ".yaml")
+        return CollectionMetadata.from_file(
+            dataset_document.csv_details.local_path.split(".")[0] + ".yaml"
+        )
 
     def _load_reader(dataset_document):
-        return PandasReader( LocalCSVAdapter.load_df(dataset_document), LocalCSVAdapter.load_metadata(dataset_document))
-
+        return PandasReader(
+            LocalCSVAdapter.load_df(dataset_document),
+            LocalCSVAdapter.load_metadata(dataset_document),
+        )
