@@ -1,37 +1,68 @@
+import importlib
 import logging
 import warnings
 import math
 import numpy as np
+from opendp.smartnoise.sql._mechanisms.accuracy import Accuracy
+from opendp.smartnoise.sql.odometer import Odometer
+from opendp.smartnoise.sql.privacy import Privacy
+
+from opendp.smartnoise.sql.reader.base import SqlReader
 from .dpsu import run_dpsu
 from .private_rewriter import Rewriter
 from .parse import QueryParser
 from .reader import PandasReader
 
-from opendp.smartnoise._ast.ast import Top
+from opendp.smartnoise._ast.ast import Query, Top
 from opendp.smartnoise._ast.expressions import sql as ast
 from opendp.smartnoise.reader import Reader
 
 from ._mechanisms.gaussian import Gaussian
-from opendp.smartnoise.report import Interval, Intervals, Result
 
 module_logger = logging.getLogger(__name__)
 
 import itertools
 
-
 class PrivateReader(Reader):
-    """Executes SQL queries against tabular data sources and returns differentially private results
+    """Executes SQL queries against tabular data sources and returns differentially private results.
+
+    PrivateReader should be created using the `from_connection` factory method.  For example,
+    using pyodbc:
+    
+    .. code-block:: python
+
+        meta = 'datasets/PUMS.yaml'
+        conn = pyodbc.connect(dsn)
+        privacy = Privacy(epsilon=0.1, delta=1/10000)
+        reader = PrivateReader.from_connection(conn, metadata=meta, privacy=privacy)
+
+        result = reader.execute('SELECT COUNT(*) AS n FROM PUMS.PUMS GROUP BY educ')
+
+    or using a Pandas dataframe:
+
+    .. code-block:: python
+
+        meta = 'datasets/PUMS.yaml'
+        csv = 'datasets/PUMS.csv'
+        pums = pd.read_csv(csv)
+
+        privacy = Privacy(epsilon=0.1, delta=1/10000)
+        reader = PrivateReader.from_connection(pums, metadata=meta, privacy=privacy)
+
+        result = reader.execute('SELECT COUNT(*) AS n FROM PUMS.PUMS GROUP BY educ')
+
     """
 
+
     def __init__(
         self,
         reader,
         metadata,
         epsilon_per_column=1.0,
         delta=10e-16,
-        interval_widths=None,
-        options=None,
-        epsilon=None,
+        *ignore,
+        privacy=None
+
     ):
         """Create a new private reader.
 
@@ -40,9 +71,6 @@ class PrivateReader(Reader):
                 The PrivateReader intercepts queries to the underlying reader and ensures differential privacy.
             :param epsilon_per_column: The privacy budget to spend for each column in the query
             :param delta: The delta privacy parameter
-            :param interval_widths: If supplied, returns confidence intervals of the specified width, e.g. [0.95, 0.75]
-            :param options: A PrivateReaderOptions with flags that change the behavior of the privacy
-                engine.
         """
         # check for old calling convention
         if isinstance(metadata, Reader):
@@ -59,49 +87,51 @@ class PrivateReader(Reader):
         else:
             raise ValueError("Parameter reader must be of type Reader")
 
-        # using string here, because we don't want to import .metadata due to circular reference
-        if "metadata.collection.CollectionMetadata" in str(type(metadata)):
-            self.metadata = metadata
-        else:
-            raise ValueError(
-                "Parameter metadata must be of type CollectionMetadata. Got {0}",
-                str(type(metadata)),
-            )
+        # we can replace this when we remove
+        # CollectionMetadata from the root __init__
+        class_ = getattr(importlib.import_module("opendp.smartnoise.metadata.collection"), "CollectionMetadata")
+        self.metadata = class_.from_(metadata)
 
         self.rewriter = Rewriter(metadata)
-        self.epsilon_per_column = epsilon_per_column
-        if epsilon is not None:
-            message = (
-                "epsilon named parameter was replaced with "
-                "epsilon_per_column to be more descriptive."
-            )
-            if epsilon != epsilon_per_column:
-                raise Exception(message)
-            else:
-                module_logger.warning(message)
-        if options is not None:
-            raise ValueError("Options has been deprecated.  Use metadata")
         self._options = PrivateReaderOptions()
 
-        self.delta = delta
-        self.interval_widths = interval_widths
+        if privacy:
+            self.privacy = privacy
+        else:
+            self.privacy = Privacy(epsilon=epsilon_per_column, delta=delta)
+        
+        self.odometer = Odometer(self.privacy)
+
         self._cached_exact = None
         self._cached_ast = None
-        self.refresh_options()
+
+        self._refresh_options()
+
+    @classmethod
+    def from_connection(cls, conn, *ignore, privacy, metadata, engine=None, **kwargs):
+        """Create a private reader over an established SQL connection.  If `engine` is not
+        passed in, the engine will be automatically detected.
+
+        :param conn: An established database connection.  Can be pyodbc, psycopg2, SparkSession, Pandas DataFrame, or Presto.
+        :param privacy:  A Privacy object with epsilon, delta, and other privacy properties.  Keyword-only.
+        :param metadata: The metadata describing the database.  `Metadata documentation is here <https://github.com/opendp/smartnoise-sdk/blob/new_opendp/sdk/Metadata.md>`_.  Keyword-only.
+        :param engine: Optional keyword-only argument that can be used to specify engine-specific rules if automatic detection fails.  This should only be necessary when using an uncommon database or middleware.
+        :returns: A `PrivateReader` object initialized to process queries against the supplied connection, using the supplied `Privacy` properties.
+        """
+        _reader = SqlReader.from_connection(conn, engine=engine, metadata=metadata, **kwargs)
+        return PrivateReader(_reader, metadata, privacy=privacy)
 
     @property
-    def epsilon(self):
-        module_logger.warning(
-            "Epsilon property will be replaced with "
-            "the more descriptive epsilon_per_column property."
-        )
-        return self.epsilon_per_column
+    def engine(self) -> str:
+        """The engine being used by this private reader.
 
-    @property
-    def engine(self):
+            df = pd.read_csv('datasets/PUMS.csv')
+            reader = PrivateReader.from_connection(df, metadata=meta, privacy=privacy)
+            assert(reader.engine == 'pandas')
+        """
         return self.reader.engine
 
-    def refresh_options(self):
+    def _refresh_options(self):
         self.rewriter = Rewriter(self.metadata)
         self.metadata.compare = self.reader.compare
         tables = self.metadata.tables()
@@ -117,12 +147,41 @@ class PrivateReader(Reader):
         self.rewriter.options.reservoir_sample = self._options.reservoir_sample
         self.rewriter.options.clamp_columns = self._options.clamp_columns
         self.rewriter.options.max_contrib = self._options.max_contrib
+        self.rewriter.options.censor_dims = self._options.censor_dims
 
-    @staticmethod
-    def get_budget_multiplier(schema, reader, query):
-        return len(PrivateReader(reader, schema, 1).get_privacy_cost(query))
+    def get_budget_multiplier(self, query_string):
+        """Analyzes the query and tells how many differentially private mechanism invocations
+         will be required to execute the query.  SUM and COUNT both use 1, while MEAN (composed of 
+         a SUM divided by a COUNT) uses 2.  VAR and STDDEV use 3.
 
-    def parse_query_string(self, query_string):
+            mul = priv.get_budget_multiplier('SELECT AVG(age) FROM PUMS.PUMS GROUP BY sex')
+            assert(mul == 2)
+
+        """
+        self._refresh_options()
+        subquery, _ = self._rewrite(query_string)
+        mechs = self._get_mechanisms(subquery)
+        return len([m for m in mechs if m])
+    
+    def get_privacy_cost(self, query_string):
+        """Estimates the epsilon and delta cost for running the given query.
+        """
+        odo = Odometer(self.privacy)
+        k = self.get_budget_multiplier(query_string)
+        odo.spend(k)
+        return odo.spent
+
+    def parse_query_string(self, query_string) -> Query:
+        """Parse a query string using this `PrivateReader`'s metadata, returning a `Query` from the AST.
+
+            reader = PrivateReader.from_connection(pums, metadata=meta, privacy=privacy)
+            query_string = 'SELECT STDDEV(age) AS age FROM PUMS.PUMS'
+            query = reader.parse_query_string(query_string)
+            age_node = query.xpath_first("//NamedExpression[@name='age']")
+            dot = age_node.visualize() # visualize the formula in the AST
+            dot.render('age', view=True, cleanup=True)
+
+        """
         queries = QueryParser(self.metadata).queries(query_string)
         if len(queries) > 1:
             raise ValueError("Too many queries provided.  We can only execute one query at a time.")
@@ -130,25 +189,24 @@ class PrivateReader(Reader):
             return []
         return queries[0]
 
-    def rewrite(self, query_string):
+    def _rewrite(self, query_string):
+        if not isinstance(query_string, str):
+            raise ValueError("Please pass a query string to _rewrite()")
         query = self.parse_query_string(query_string)
-        return self.rewrite_ast(query)
+        return self._rewrite_ast(query)
 
-    def rewrite_ast(self, query):
+    def _rewrite_ast(self, query):
+        if isinstance(query, str):
+            raise ValueError("Please pass a Query AST object to _rewrite_ast()")
         query_max_contrib = query.max_ids
         if self._options.max_contrib is None or self._options.max_contrib > query_max_contrib:
             self._options.max_contrib = query_max_contrib
 
-        self.refresh_options()
+        self._refresh_options()
         query = self.rewriter.query(query)
         subquery = query.source.relations[0].primary.query
         return (subquery, query)
 
-    def get_privacy_cost(self, query_string):
-        self.refresh_options()
-        subquery, query = self.rewrite(query_string)
-        return subquery.numeric_symbols()
-
     def _get_reader(self, query_ast):
         if (
             query_ast.agg is not None
@@ -156,52 +214,19 @@ class PrivateReader(Reader):
             and isinstance(self.reader, PandasReader)
         ):
             query = str(query_ast)
-            dpsu_df = run_dpsu(self.metadata, self.reader.df, query, eps=1.0)
+            dpsu_df = run_dpsu(self.metadata, self.reader.df, query, epsilon=1.0)
             return PandasReader(dpsu_df, self.metadata)
         else:
             return self.reader
 
-    def execute(self, query_string):
-        """Executes a query and returns a recordset that is differentially private.
-
-        Follows ODBC and DB_API convention of consuming query as a string and returning
-        recordset as tuples.  This is useful for cases where existing DB_API clients
-        want to swap out API calls with minimal changes.
-
-        :param query_string: A query string in SQL syntax
-        :return: A recordset structured as an array of tuples, where each tuple
-         represents a row, and each item in the tuple is typed.  The first row should
-         contain column names.
-        """
-        query = self.parse_query_string(query_string)
-        return self._execute_ast(query)
-
-    def _execute_ast(self, query, cache_exact=False):
-        if isinstance(query, str):
-            raise ValueError("Please pass AST to _execute_ast.")
-
-        subquery, query = self.rewrite_ast(query)
+    def _get_mechanisms(self, subquery: Query):
         max_contrib = self._options.max_contrib if self._options.max_contrib is not None else 1
-        thresh_scale = math.sqrt(max_contrib) * (
-            (
-                math.sqrt(math.log(1 / self.delta))
-                + math.sqrt(math.log(1 / self.delta) + self.epsilon_per_column)
-            )
-            / (math.sqrt(2) * self.epsilon_per_column)
-        )
-        self.tau = 1 + thresh_scale * math.sqrt(
-            2 * math.log(max_contrib / math.sqrt(2 * math.pi * self.delta))
-        )
 
         syms = subquery.all_symbols()
-        source_col_names = [s[0] for s in syms]
 
         # list of sensitivities in column order
         sens = [s[1].sensitivity() for s in syms]
 
-        # tell which are counts, in column order
-        is_count = [s[1].is_count for s in syms]
-
         # set sensitivity to None if the column is a grouping key
         if subquery.agg is not None:
             group_keys = [
@@ -214,28 +239,73 @@ class PrivateReader(Reader):
         for idx in range(len(sens)):
             if is_group_key[idx]:
                 sens[idx] = None
-
         if any([s is np.inf for s in sens]):
             raise ValueError(
                 "Query is attempting to query an unbounded column that isn't part of the grouping key"
             )
-
-        kc_pos = None
-        kcc_pos = []
-        for idx in range(len(syms)):
-            sname, sym = syms[idx]
-            if sname == "keycount":
-                kc_pos = idx
-            elif sym.is_key_count:
-                kcc_pos.append(idx)
-        if kc_pos is None and len(kcc_pos) > 0:
-            kc_pos = kcc_pos.pop()
-
-        # make a list of mechanisms in column order
         mechs = [
-            Gaussian(self.epsilon_per_column, self.delta, s, max_contrib) if s is not None else None
+            Gaussian(self.privacy.epsilon, self.privacy.delta, s, max_contrib) if s is not None else None
             for s in sens
         ]
+        return mechs
+    def execute_with_accuracy(self, query_string:str):
+        return self.execute(query_string, accuracy=True)
+
+    def execute_with_accuracy_df(self, query_string:str, *ignore, privacy:bool=False):
+        return self.execute_df(query_string, accuracy=True)
+
+
+    def execute(self, query_string, accuracy:bool=False):
+        """Executes a query and returns a recordset that is differentially private.
+
+        Follows ODBC and DB_API convention of consuming query as a string and returning
+        recordset as tuples.  This is useful for cases where existing DB_API clients
+        want to swap out API calls with minimal changes.
+
+        :param query_string: A query string in SQL syntax
+        :return: A recordset structured as an array of tuples, where each tuple
+         represents a row, and each item in the tuple is typed.  The first row should
+         contain column names.
+        """
+        query = self.parse_query_string(query_string)
+        return self._execute_ast(query, accuracy=accuracy)
+
+    def _execute_ast(self, query, *ignore, accuracy:bool=False, cache_exact=False):
+        if isinstance(query, str):
+            raise ValueError("Please pass AST to _execute_ast.")
+
+        subquery, query = self._rewrite_ast(query)
+        max_contrib = self._options.max_contrib if self._options.max_contrib is not None else 1
+
+        _accuracy = None
+        if accuracy:
+            _accuracy = Accuracy(query, subquery, self.privacy)
+
+        thresh_scale = math.sqrt(max_contrib) * (
+            (
+                math.sqrt(math.log(1 / self.privacy.delta))
+                + math.sqrt(math.log(1 / self.privacy.delta) + self.privacy.epsilon)
+            )
+            / (math.sqrt(2) * self.privacy.epsilon)
+        )
+        self.tau = 1 + thresh_scale * math.sqrt(
+            2 * math.log(max_contrib / math.sqrt(2 * math.pi * self.privacy.delta))
+        )
+
+        syms = subquery.all_symbols()
+        source_col_names = [s[0] for s in syms]
+
+        # tell which are counts, in column order
+        is_count = [s[1].is_count for s in syms]
+
+        kc_pos = None
+        for idx in range(len(syms)):
+            sname, _ = syms[idx]
+            if sname == "keycount":
+                kc_pos = idx
+
+        # get a list of mechanisms in column order
+        mechs = self._get_mechanisms(subquery)
 
         # execute the subquery against the backend and load in tuples
         if cache_exact:
@@ -243,6 +313,7 @@ class PrivateReader(Reader):
             if self._cached_exact is not None:
                 if subquery == self._cached_ast:
                     db_rs = self._cached_exact
+                    _accuracy = self._cached_accuracy
                 else:
                     raise ValueError(
                         "Cannot run different query against cached result.  "
@@ -252,9 +323,10 @@ class PrivateReader(Reader):
                 db_rs = self._get_reader(subquery)._execute_ast(subquery)
                 self._cached_exact = list(db_rs)
                 self._cached_ast = subquery
+                self._cached_accuracy = _accuracy
         else:
-            self.cached_exact = None
-            self.cached_ast = None
+            self._cached_exact = None
+            self._cached_ast = None
             db_rs = self._get_reader(subquery)._execute_ast(subquery)
 
         clamp_counts = self._options.clamp_counts
@@ -264,16 +336,14 @@ class PrivateReader(Reader):
             row = [v for v in row_in]
             # set null to 0 before adding noise
             for idx in range(len(row)):
-                if sens[idx] is not None and row[idx] is None:
+                #if sens[idx] is not None and row[idx] is None:
+                if mechs[idx] and row[idx] is None:
                     row[idx] = 0.0
             # call all mechanisms to add noise
             out_row = [
                 noise.release([v]).values[0] if noise is not None else v
                 for noise, v in zip(mechs, row)
             ]
-            # ensure all key counts are the same
-            for idx in kcc_pos:
-                out_row[idx] = out_row[kc_pos]
             # clamp counts to be non-negative
             if clamp_counts:
                 for idx in range(len(row)):
@@ -290,7 +360,10 @@ class PrivateReader(Reader):
         else:
             out = map(process_row, db_rs[1:])
 
+        # censor infrequent dimensions
         if subquery.agg is not None and self._options.censor_dims:
+            if kc_pos == None:
+                raise ValueError("Query needs a key count column to censor dimensions")
             if hasattr(out, "filter"):
                 # it's an RDD
                 tau = self.tau
@@ -317,11 +390,21 @@ class PrivateReader(Reader):
                     return bool(str(val).replace('"', "").replace("'", ""))
             else:
                 raise ValueError("Can't convert type " + type)
+        
+
+        alphas = [alpha for alpha in self.privacy.alphas]
 
         def process_out_row(row):
             bindings = dict((name.lower(), val) for name, val in zip(source_col_names, row))
-            row = [c.expression.evaluate(bindings) for c in query.select.namedExpressions]
-            return [convert(val, type) for val, type in zip(row, out_types)]
+            out_row = [c.expression.evaluate(bindings) for c in query.select.namedExpressions]
+            out_row =[convert(val, type) for val, type in zip(out_row, out_types)]
+
+            # compute accuracies
+            if accuracy == True and alphas:
+                accuracies = [_accuracy.accuracy(row=list(row), alpha=alpha) for alpha in alphas]
+                return tuple([out_row, accuracies])
+            else:
+                return tuple([out_row, []])
 
         if hasattr(out, "map"):
             # it's an RDD
@@ -330,7 +413,7 @@ class PrivateReader(Reader):
             out = map(process_out_row, out)
 
         def filter_aggregate(row, condition):
-            bindings = dict((name.lower(), val) for name, val in zip(out_col_names, row))
+            bindings = dict((name.lower(), val) for name, val in zip(out_col_names, row[0]))
             keep = condition.evaluate(bindings)
             return keep
 
@@ -365,17 +448,19 @@ class PrivateReader(Reader):
                 sort_fields.append(sf)
 
             def sort_func(row):
-                return tuple(
+                row_value, accuracies = row
+                out_row_value = tuple(
                     [
-                        row[idx]
+                        row_value[idx]
                         if not desc
-                        else not row[idx]
+                        else not row_value[idx]
                         if out_types[idx] == "boolean"
-                        else -row[idx]
+                        else -row_value[idx]
                         for desc, idx in sort_fields
                     ]
                 )
-
+                return tuple([out_row_value, accuracies])
+                
             if hasattr(out, "sortBy"):
                 out = out.sortBy(sort_func)
             else:
@@ -399,19 +484,35 @@ class PrivateReader(Reader):
                 else:
                     out = itertools.islice(out, limit_rows)
 
+        # drop empty accuracy if no accuracy requested
+        def drop_accuracy(row):
+            return row[0]
+        if accuracy == False:
+            if hasattr(out, "map"):
+                # it's an RDD
+                out = out.map(drop_accuracy)
+            else:
+                out = map(drop_accuracy, out)
+
+        # increment odometer
+        self.odometer.spend(len([m for m in mechs if m]))
+
         # output it
-        if hasattr(out, "toDF"):
+        if accuracy == False and hasattr(out, "toDF"):
             # Pipeline RDD
             return out.toDF(out_col_names)
         elif hasattr(out, "map"):
             # Bare RDD
             return out
         else:
-            out_rows = [out_col_names] + list(out)
+            row0 = [out_col_names]
+            if accuracy == True:
+                row0 = [[out_col_names, [[col_name+'_' + str(1-alpha).replace('0.', '') for col_name in out_col_names] for alpha in self.privacy.alphas ]]]
+            out_rows = row0 + list(out)
             return out_rows
 
     def _execute_ast_df(self, query, cache_exact=False):
-        return self._to_df(self._execute_ast(query, cache_exact))
+        return self._to_df(self._execute_ast(query, cache_exact=cache_exact))
 
 
 class PrivateReaderOptions:
