@@ -1,4 +1,4 @@
-from .base import SqlReader, NameCompare
+from .base import Serializer, SqlReader, NameCompare
 from .engine import Engine
 
 from opendp.smartnoise._ast.tokens import Literal
@@ -8,30 +8,25 @@ from opendp.smartnoise._ast.expressions.numeric import BareFunction
 class SparkReader(SqlReader):
     ENGINE = Engine.SPARK
 
-    def __init__(self, session):
-        super().__init__(SparkNameCompare(), SparkSerializer())
-        from pyspark.sql import SparkSession  # TODO how do we deal with reader dependencies
+    def __init__(self, conn, **kwargs):
+        super().__init__(self.ENGINE)
 
-        self.api = session
+        self.api = conn
         self.database = "Spark Session"
 
-    def execute(self, query):
+    def execute(self, query, *ignore, accuracy:bool=False):
         if not isinstance(query, str):
             raise ValueError("Please pass strings to execute.  To execute ASTs, use execute_typed.")
-
         return self.api.sql(query)
 
     def _to_df(rows):
         return rows
 
-    def execute_typed(self, query):
-        return self.execute(query)
-
     def db_name(self):
         return self.database
 
 
-class SparkSerializer:
+class SparkSerializer(Serializer):
     def serialize(self, query):
         for r_e in [n for n in query.find_nodes(BareFunction) if n.name == "RANDOM"]:
             r_e.name = "rand"
